-------------------------------------------------------------------------------
-- Title      :  FPU Calculation units
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : Cal_unit.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/12/12
-- Last update : 2000/12/12
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164,ieee.std_logic_unsigned
--               fpu.fpu_pkg, fpu.components_pkg
--				 utility.tools_pkg
-------------------------------------------------------------------------------
-- Description: FPU calculation units
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   12 Dec 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :   
-- To Optimize
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

library utility;
use utility.tools_pkg.all;

library FPU;
use FPU.fpu_pkg.all;
use FPU.components_pkg.all;

entity cal_unit_ent is

  port (
    clk       : in  std_logic;          -- system clock
    rst       : in  std_logic;          -- system reset
    Operand1  : in  FPinternal_typ;     -- Operand 1
    Operand2  : in  FPinternal_typ;     -- Operand 2
    NoType1   : in  SPECIALNO_typ;      -- No. 1 type
    NoType2   : in  SPECIALNO_typ;      -- No. 2 type
    ResType   : out SPECIALNO_typ;      -- Result Special no. type
    Result    : out FPRESULT_typ;       -- Result
    load      : in  std_logic;          -- Load new operands
    done      : out std_logic;          -- calculations done
    Operation : in  std_logic_vector(OPERATIONS_WIDTH-1 downto 0));  -- Operation
--    Operation : in  std_logic_vector(1 downto 0));  -- Operation

end cal_unit_ent;

architecture cal_unit_beh of cal_unit_ent is

  signal addsub_op  : std_logic;        -- Add Subtract operation select
  signal AddResult  : FPRESULT_typ;     -- Add/Subtract unit Result
  signal ADDResType : SPECIALNO_typ;    --  Add/Subtract result type
begin  -- cal_unit_beh

-------------------------------------------------------------------------------
  addsub_unit      : addsub_ent
    port map (
      Operand1  => Operand1,
      Operand2  => Operand2,
      NoType1   => NoType1,
      NoType2   => NoType2,
      Result    => AddResult,
      clk       => clk,
      rst       => rst,
      ResType   => ADDResType,
      Operation => addsub_op);
-------------------------------------------------------------------------------
-- purpose: Select calculation unit
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  select_unit_proc : process (clk, rst)

    --variable select_res : std_logic_vector(log2(OPERATIONS)-1 downto 0);
                                        -- Select result
    variable counter    : std_logic_vector(log2(MAX_CYCLES)-1 downto 0);
                                        -- Calculation counter
    variable NoCycles   : std_logic_vector(log2(MAX_CYCLES)-1 downto 0);
                                        -- No. of cycles needed to the operation


  begin  -- process select_unit_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      Result.fract <= (others => '0');
      Result.exp   <= (others => '0');
      Result.sign  <= '0';

      done <= '0';

      ResType.inf  <= '0';
      ResType.ZERO <= '0';
      ResType.SNAN <= '0';
      ResType.QNAN <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge
      -- Result Mux select 
      case Operation is
        when ADD_OP =>
          addsub_op <= '0';

          Result.fract <= AddResult.fract;
          Result.exp   <= AddResult.exp;
          Result.sign  <= AddResult.sign;

          ResType.inf  <= ADDResType.inf;
          ResType.ZERO <= ADDResType.ZERO;
          ResType.SNAN <= ADDResType.SNAN;
          ResType.QNAN <= ADDResType.QNAN;

          NoCycles := int_2_slv(ADD_CYCLES-1,log2(MAX_CYCLES));

        when SUB_OP =>
          addsub_op <= '1';

          Result.fract <= AddResult.fract;
          Result.exp   <= AddResult.exp;
          Result.sign  <= AddResult.sign;

          NoCycles := int_2_slv(SUB_CYCLES-1,log2(MAX_CYCLES));

          ResType.inf  <= ADDResType.inf;
          ResType.ZERO <= ADDResType.ZERO;
          ResType.SNAN <= ADDResType.SNAN;
          ResType.QNAN <= ADDResType.QNAN;

        when others               =>
          -- unsupported operation 
          Result.fract <= (others => '1');
          Result.exp   <= (others => '1');
          Result.sign  <= '1';
          NoCycles     := int_2_slv(OVERHEAD_CYCLES-1,log2(MAX_CYCLES));

          ResType.inf  <= '0';
          ResType.ZERO <= '0';
          ResType.SNAN <= '1';
          ResType.QNAN <= '0';
      end case;

      if load = '1' then
        counter               := (others => '0');
        done                  <= '0';
      elsif counter = NoCycles then
        counter               := (others => '0');
                    done      <= '1';
                    else
                      counter := counter + 1;
                      done    <= '0';
                    end if;

                    end if;
                    end process select_unit_proc;
                    end cal_unit_beh;
