-------------------------------------------------------------------------------
-- Title      :  FPU Top level
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : FpuTop.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/12/12
-- Last update : 2000/12/12
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164
--               utility.tools_pkg
--               fpu.fpu_pkg, fpu.components_pkg
-------------------------------------------------------------------------------
-- Description: FPU top level core
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   12 Dec 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :   
-- To Optimize
-------------------------------------------------------------------------------
library fpu;
use fpu.fpu_pkg.all;
use fpu.components_pkg.all;

library ieee;
use ieee.std_logic_1164.all;

library utility;
use utility.tools_pkg.all;


entity fpu_ent is

  generic (
    FP_PRECISION : integer := SINGLE_PRECISION);  -- FPU precision

  port (
    Operand1   : in  FPNUMBER_typ;      -- Operand 1
    Operand2   : in  FPNUMBER_typ;      -- Operand 2
    clk        : in  std_logic;         -- system clock
    rst        : in  std_logic;         -- system reset
    Round_Mode : in  std_logic_vector(1 downto 0);  -- Round mode
    Result     : out FPNUMBER_typ;      -- Result
    Exceptions : out EXCEPTION_typ;     -- FPU exceptions
    conditions : out CONDITIONS_typ;    -- FP operation conditions
    load       : in  std_logic;         -- Load new operands
    done       : out std_logic;         -- calculations done
    Operation  : in  std_logic_vector(OPERATIONS_WIDTH-1 downto 0));  -- Operation

end fpu_ent;


architecture fpu_str of fpu_ent is
  signal decoded_op1 : FPinternal_typ;  -- decoded Operand 1
  signal decoded_op2 : FPinternal_typ;  -- decoded Operand 2

  signal decoded_type1 : SPECIALNO_typ;  -- decoded special no. type 1
  signal decoded_type2 : SPECIALNO_typ;  -- decoded special no. type 2

  signal Cal_res : FPRESULT_typ;        -- Calculated Exact Result

  signal Round_UP : std_logic;          -- Round up indicator
  signal DivZero  : std_logic;          -- Divide by zero indicator

  signal Post_Round_res  : FPinternal_typ;   -- Post Rounded Result
  signal Post_Round_Type : SPECIALNO_typ;  -- Post Round result type

begin  -- fpu_str
-------------------------------------------------------------------------------
  decode1        : decode_ent
    port map (
      InNumber    => Operand1,
      OutNumber   => decoded_op1,
      clk         => clk,
      rst         => rst,
      NoType      => decoded_type1);
-------------------------------------------------------------------------------
  decode2        : decode_ent
    port map (
      InNumber    => Operand2,
      OutNumber   => decoded_op2,
      clk         => clk,
      rst         => rst,
      NoType      => decoded_type2);
-------------------------------------------------------------------------------
  calculation    : cal_unit_ent
    port map (
      clk         => clk,
      rst         => rst,
      Operand1    => decoded_op1,
      Operand2    => decoded_op2,
      NoType1     => decoded_type1,
      NoType2     => decoded_type2,
      ResType     => Post_round_type,
      Result      => Cal_res,
      load        => load,
      done        => done,
      Operation   => Operation);
-------------------------------------------------------------------------------
  round_Norm     : rndnorm_ent
    port map (
      clk         => clk,
      rst         => rst,
      roundmode   => Round_Mode,
      ResIn       => Cal_res,
      Inexact_exp => Round_up,
      Result      => Post_Round_res);
------------------------------------------------------------------------------ 
  exceptions_gen : excp_gen_ent
    port map (
      clk         => clk,
      rst         => rst,
      inexact     => Round_up,
      DivZero     => DivZero,
      ResType     => Post_Round_type,
      Result      => Post_Round_res,
      Exceptions  => Exceptions);
-------------------------------------------------------------------------------
  encode_res     : encode_ent
    port map (
      clk         => clk,
      rst         => rst,
      RoundedRes  => Post_Round_Res,
      Result      => Result,
      ResType     => Post_Round_Type);
-------------------------------------------------------------------------------
end fpu_str;
