-------------------------------------------------------------------------------
-- Title      :  Adder Subtractor core
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : addsub.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/11/5
-- Last update : 2000/11/5
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164
--               fpu.fpu_pkg
-------------------------------------------------------------------------------
-- Description: Adder subtractor core
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   5th Nov 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :
-- To Optimze      : 1) Find a beter way in exponent diff and Operand shift (swap)
--                                       2) The shifter needs high fan outs
-------------------------------------------------------------------------------


library FPU;
use FPU.fpu_pkg.all;

library utility;
use utility.tools_pkg.all;
use utility.cores_pkg.all;


library ieee;
use ieee.std_logic_1164.all;

entity addsub_ent is

  port (
    Operand1  : in  FPinternal_typ;     -- Operand 1
    Operand2  : in  FPinternal_typ;     -- Operand2
    NoType1   : in  SPECIALNO_typ;      -- No.1 Type
    NoType2   : in  SPECIALNO_typ;      -- No. 2 Type
    Result    : out FPRESULT_typ;       -- Result no.
    Operation : in  std_logic;          -- Operation '1':Subtract / '0':Add 
    clk       : in  std_logic;          -- System clock
    rst       : in  std_logic;          -- System Reset
    ResType   : out SPECIALNO_typ);     -- Result special no. detection

end addsub_ent;

architecture addsub_beh of addsub_ent is
-------------------------------------------------------------------------------
-- Phases
-- 1: Calculate the Exponent Diff E1-E2, invert the sign if sub operation and
-- register the inputs
-- 2: store operands and do the alignment
-- 3: Expand the fraction precession and do the addition
-------------------------------------------------------------------------------
-- Phase 1 signals


  signal Diff          : std_logic_vector(6 downto 0);  -- Exponents Difference
-- The maximum diff size is a result of the maximum no of shifts  the fraction
-- can handle = 64 bit
  signal SelectSmallOp : std_logic;     -- Select Small Operand to shift
  signal SmallOp       : std_logic_vector(Operand2.fract'length -1 +39 downto 0);  -- Small Operand to shift

  -- Operand registers
  -- extended fract length is 24 (IEEE representation)+ carry bit+ 39(extra bits "Infinity
  -- precision") = 64
  -- 
  signal Op1_ph1_fract : std_logic_vector(Operand1.fract'length -1 +39 downto 0);
                                        -- Extended size of the fraction
  signal Op1_ph1_exp   : std_logic_vector(Operand1.exp'length -1 downto 0);
                                        -- Exponent register of Operand1
  signal Op1_ph1_sign  : std_logic;     -- sign register of Operand1

  signal Op2_ph1_fract : std_logic_vector(Operand2.fract'length -1 +39 downto 0);
                                        -- Extended size of the fraction Operand2
  signal Op2_ph1_exp   : std_logic_vector(Operand1.exp'length -1 downto 0);
                                        -- Exponent register of Operand2
  signal Op2_ph1_sign  : std_logic;     -- sign register of Operand2

-- Phase 2 signals
  -- Operand registers
  signal Op1_ph2_fract : std_logic_vector(Operand1.fract'length -1 +39 downto 0);
                                        -- Extended size of the fraction
  signal Op1_ph2_exp   : std_logic_vector(Operand1.exp'length -1 downto 0);
                                        -- Exponent register of Operand1
  signal Op1_ph2_sign  : std_logic;     -- sign register of Operand1

  signal Op2_ph2_fract : std_logic_vector(Operand2.fract'length -1 +39 downto 0);
                                        -- Extended size of the fraction Operand2
  signal Op2_ph2_exp   : std_logic_vector(Operand1.exp'length -1 downto 0);
                                        -- Exponent register of Operand2
  signal Op2_ph2_sign  : std_logic;     -- sign register of Operand2

  signal SmallOpShifted : std_logic_vector(Operand2.fract'length -1 +39 downto 0);

-- Phase 3 signals
  
-- Special no. processor signals
  signal ResType1 : SPECIALNO_typ;      -- Result no type reg1
  signal ResType2 : SPECIALNO_typ;      -- Result no type reg2
-----------------------------------------------------------------------------  
begin  -- addsub_beh

-------------------------------------------------------------------------------
-- purpose: Calculate the exponent Difference between FP numbers
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  calcdiff_proc : process (clk, rst)

    variable ExpDiff         : SIGN_MAG_typ(Operand1.exp'length downto 0);  -- Exponent Difference
    variable LargeDiffDetect : std_logic;  -- Large Difference detect

  begin  -- process calcdiff_proc

    if rst = '0' then                   -- asynchronous reset (active low)

      LargeDiffDetect := '0';
      ExpDiff         := (others => '0');
      Diff            <= (others => '0');

      Op1_ph1_sign  <= '0';
      Op1_ph1_exp   <= (others => '0');
      Op1_ph1_fract <= (others => '0');

      Op2_ph1_sign  <= '0';
      Op2_ph1_exp   <= (others => '0');
      Op2_ph1_fract <= (others => '0');

      SelectSmallOp <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge
      -- This must be optimized so as not to do twice complement caused by Sign_mag operations

      ExpDiff := SIGN_MAG_typ('0' & Operand1.exp) - SIGN_MAG_typ('0' & Operand2.exp);

      -- Check the sign of the result
      SelectSmallOp <= std_logic(ExpDiff(ExpDiff'length-1));

      LargeDiffDetect := '0';

      -- check if the difference is larger than fraction size
      for i in ExpDiff'length -2 downto Diff'length loop

        LargeDiffDetect := LargeDiffDetect or ExpDiff(i);

      end loop;  -- i

      -- set the shift size (if larger than 64 set it to maximum to flush the fraction)
      if LargeDiffDetect = '1' then
        Diff <= (others => '1');        -- Shift out the fraction
      else
        Diff <= std_logic_vector(ExpDiff(Diff'length -1 downto 0));
      end if;


-- registering the inputs and increase the width of the fraction.
      Op1_ph1_sign  <= Operand1.sign;
      Op1_ph1_exp   <= Operand1.exp;
      Op1_ph1_fract <= Operand1.fract & ( Op1_ph1_fract'length - Operand2.fract'length-1 downto 0 => '0');


      Op2_ph1_sign  <= Operand2.sign xor Operation;  --If operation is - invert the sign bit
      Op2_ph1_exp   <= Operand2.exp;
      Op2_ph1_fract <= Operand2.fract & ( Op1_ph2_fract'length - Operand2.fract'length-1 downto 0 => '0');

    end if;
  end process calcdiff_proc;
-------------------------------------------------------------------------------
-- Align phase

  smallOp <= op2_ph1_fract when SelectSmallOp = '0' else op1_ph1_fract;  -- select the small


  shifter : BarrelShifter
    generic map (
      REGSIZE   => SmallOp'length,
      DIRECTION => 0)                   --shift right
    port map (
      inReg     => SmallOp,
      ShSize    => Diff,
      outReg    => smallOpShifted );

-------------------------------------------------------------------------------
-- purpose: Register the perands and possible swap
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  reg_ops : process (clk, rst)
  begin  -- process reg_ops
    if rst = '0' then                   -- asynchronous reset (active low)

      Op1_ph2_sign  <= '0';
      Op1_ph2_exp   <= (others => '0');
      Op1_ph2_fract <= (others => '0');

      Op2_ph2_sign  <= '0';
      Op2_ph2_exp   <= (others => '0');
      Op2_ph2_fract <= (others => '0');

    elsif clk'event and clk = '1' then  -- rising clock edge

-- set sign
      op1_ph2_sign <= op1_ph1_sign;
      op2_ph2_sign <= op2_ph1_sign;

-- set the exponents
-- This uses lot of mux's; must be improved
      if SelectSmallOp = '0' then       -- Operand1 is the larger

        op2_ph2_exp <= op1_ph1_exp;
        op1_ph2_exp <= op1_ph1_exp;

        op2_ph2_fract <= smallOpShifted;
        op1_ph2_fract <= op1_ph1_fract;

      else                              -- Operand2 is the larger

        op2_ph2_exp <= op2_ph1_exp;
        op1_ph2_exp <= op2_ph1_exp;

        op1_ph2_fract <= smallOpShifted;
        op2_ph2_fract <= op2_ph1_fract;

      end if;



    end if;
  end process reg_ops;
-------------------------------------------------------------------------------
  -- purpose: Perform the addition
  -- type   : sequential
  -- inputs : clk, rst
  -- outputs: 
  adder : process (clk, rst)

    variable res : SIGN_MAG_typ(Op1_ph1_fract'length downto 0);  -- result


  begin  -- process adder
    if rst = '0' then
      -- asynchronous reset (active low)

      res := (others => '0');

      result.sign  <= '0';
      result.fract <= (others => '0');
      result.exp   <= (others => '0');


    elsif clk'event and clk = '1' then
      -- rising clock edge
      res := SIGN_MAG_typ(Op1_ph2_sign & Op1_ph2_fract)
             +
             SIGN_MAG_typ(Op2_ph2_sign & Op2_ph2_fract);

      result.sign  <= res(res'length-1);
      result.fract <= std_logic_vector(res(res'length-2 downto 0));
      result.exp   <= op1_ph2_exp;



    end if;
  end process adder;

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Special no. are handled separetly and must be delayed to get out with the
-- same timing of the overall calculation.
-------------------------------------------------------------------------------
-- purpose: register special no status
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  register1_specail : process (clk, rst)
  begin  -- process register1_specail
    if rst = '0' then                   -- asynchronous reset (active low)
      ResType1.QNAN <= '0';
      ResType1.SNAN <= '0';
      ResType1.INF  <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge

      ResType1.QNAN <= NoType1.QNAN or NoType2.QNAN;
      ResType1.SNAN <= (Operand1.sign xor Operand2.sign) and NoType1.INF and NoType2.INF;
      ResType1.INF  <= NoType1.INF or NoType2.INF;


    end if;

  end process register1_specail;
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- purpose: register special no status
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  register2_specail : process (clk, rst)
  begin  -- process register1_specail
    if rst = '0' then                   -- asynchronous reset (active low)
      ResType2.QNAN <= '0';
      ResType2.SNAN <= '0';
      ResType2.INF  <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge

      ResType2.QNAN <= ResType1.QNAN;
      ResType2.SNAN <= ResType1.SNAN;
      ResType2.INF  <= ResType1.INF;


    end if;

  end process register2_specail;
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- purpose: register special no status
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  register3_specail : process (clk, rst)
  begin  -- process register1_specail
    if rst = '0' then                   -- asynchronous reset (active low)
      ResType.QNAN <= '0';
      ResType.SNAN <= '0';
      ResType.INF  <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge

      ResType.QNAN <= ResType2.QNAN;
      ResType.SNAN <= ResType2.SNAN;
      ResType.INF  <= ResType2.INF;


    end if;

  end process register3_specail;
-------------------------------------------------------------------------------


end addsub_beh;
