-------------------------------------------------------------------------------
-- Title      :  Decode core
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : decode.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/11/3
-- Last update : 2000/11/3
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164
--               fpu.fpu_pkg
-------------------------------------------------------------------------------
-- Description: FPU Decode: extracts the special numbers and the hidden bit
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   3rd Nov 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Bugs            :
--                     1) Sticky and other bits are never used before the calculation
-------------------------------------------------------------------------------


library FPU;
use FPU.fpu_pkg.all;

library ieee;
use ieee.std_logic_1164.all;

entity Decode_ent is

  port (
    InNumber  : in  FPNUMBER_typ;       -- FP number input
    OutNumber : out FPinternal_typ;     -- decoded fp number
    clk       : in  std_logic;          -- system clock
    rst       : in  std_logic;          -- system reset
    NoType    : out SPECIALNO_typ );    -- Special no. result

end Decode_ent;

-------------------------------------------------------------------------------
architecture Decode_beh of Decode_ent is

begin  -- Decode_beh
-------------------------------------------------------------------------------
  -- purpose: Decodes the number and recovers the hidden bit for normalized and denormalized numbers
  -- type   : sequential
  -- inputs : clk, rst
  -- outputs: 
  decode_proc : process (clk, rst)

    variable HiddenBit : std_logic;     -- Hidden bit

  begin  -- process decode_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      OutNumber.sign  <= '0';
      OutNumber.exp   <= (others => '0');
      OutNumber.fract <= (others => '0');

      HiddenBit := '0';

    elsif clk'event and clk = '1' then  -- rising clock edge

      for i in InNumber.exp 'range loop

        HiddenBit := HiddenBit or InNumber.exp(i);

      end loop;  -- i

      OutNumber.fract <= '0' & HiddenBit & InNumber.fract;

      OutNumber.sign <= InNumber.sign;

      OutNumber.exp <= '0' & InNumber.exp;


    end if;
  end process decode_proc;

-------------------------------------------------------------------------------
-- purpose: Decodes Special numbers
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  GetSpecial_proc : process (clk, rst)

    variable zero_fract : std_logic;    -- Fraction is all zero
    variable one_fract  : std_logic;    -- Fraction is all one
    variable zero_exp   : std_logic;    -- Exponent is all zero
    variable one_exp    : std_logic;    -- Exponent is all one

  begin  -- process GetSpecial_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      NoType.INF  <= '0';
      NoType.ZERO <= '0';
      NoType.SNAN <= '0';
      NoType.QNAN <= '0';

      zero_fract := '0';
      one_fract  := '0';
      zero_exp   := '0';
      one_exp    := '0';

    elsif clk'event and clk = '1' then  -- rising clock edge


      for i in InNumber.fract'range loop

        zero_fract := zero_fract or InNumber.fract(i);  -- if all are zero =>
                                                        -- zero_fract = '0'

        one_fract := one_fract and InNumber.fract(i);  -- if all are one =>
                                                       -- one_fract = '1'

      end loop;  -- i

      zero_fract := not zero_fract;

      for i in InNumber.exp'range loop

        zero_exp := zero_exp or InNumber.exp(i);  -- if all are zero => 
                                                  -- zero_exp = '0'

        one_exp := one_exp and InNumber.exp(i);  -- if all are one =>
                                                 --  one_fract = '1'

      end loop;  -- i in InNumber.exp'range

      zero_exp := not zero_exp;

------------------     
      NoType.INF  <= (zero_fract) and (one_exp);
      NoType.ZERO <= (zero_fract) and (zero_exp);
      NoType.QNAN <= (not zero_fract) and (one_exp);
      NoType.SNAN <= '0';

-----    
    end if;
  end process GetSpecial_proc;
-------------------------------------------------------------------------------

end Decode_beh;
