-------------------------------------------------------------------------------
-- Title      :  Exceptions generator core
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : excp_gen.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/12/12
-- Last update : 2000/12/12
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164
--               utility.tools_pkg
--               fpu.fpu_pkg
-------------------------------------------------------------------------------
-- Description: Exceptions generator core
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   12 Dec 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :   Invalid exception signal becomes valid before other exceptions
--					   because it is derived from the calculation units and not the 
--					   round normalize unit
-- To Optimize
-------------------------------------------------------------------------------


library FPU;
use FPU.fpu_pkg.all;

library utility;
use utility.tools_pkg.all;


library ieee;
use ieee.std_logic_1164.all;

entity excp_gen_ent is

  port (
    clk        : in  std_logic;         -- System clock
    rst        : in  std_logic;         -- System Reset
    inexact    : in  std_logic;         -- Inexact result
    DivZero : in std_logic;             -- Divided by zero
    ResType    : in  SPECIALNO_typ;     -- Result Type
    Result     : in  FPinternal_typ;    -- Result no.
    Exceptions : out EXCEPTION_typ);    -- Exceptions

end excp_gen_ent;

architecture excp_gen_beh of excp_gen_ent is

begin  -- excp_gen_beh
-------------------------------------------------------------------------------
  -- purpose: Inexact exception generator
  -- type   : sequential
  -- inputs : clk, rst
  -- outputs: 
  inexact_div_proc: process (clk, rst)
  begin  -- process inexact_proc
    if rst = '0' then                   -- asynchronous reset (active low)
      Exceptions.inexact <= '0';
      Exceptions.DivideZero <= '0';
      
    elsif clk'event and clk = '1' then  -- rising clock edge

        Exceptions.inexact <= Inexact;
        Exceptions.DivideZero <= DivZero;
        
    end if;
  end process inexact_div_proc;
-------------------------------------------------------------------------------
-- purpose: Underflow exception generator
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
underflow_proc: process (clk, rst)
  variable temp : std_logic;            -- Temporary variable
begin  -- process underflow_proc
  if rst = '0' then                     -- asynchronous reset (active low)

    Exceptions.underflow <= '0';
    temp := '0';
    
  elsif clk'event and clk = '1' then    -- rising clock edge
    temp := '0';
    for i in Result.exp'range loop
      
      temp := temp or Result.exp(i);
      
    end loop;  -- i

    Exceptions.underflow <= not temp;
      
  end if;
end process underflow_proc;
-------------------------------------------------------------------------------
-- purpose: Overflow exception generator
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
overflow_proc: process (clk, rst)
begin  -- process overflow_proc
  if rst = '0' then                     -- asynchronous reset (active low)

    Exceptions.overflow <= '0';
    
  elsif clk'event and clk = '1' then    -- rising clock edge

    Exceptions.overflow <= Result.exp(Result.exp'length-1);
    
  end if;
end process overflow_proc;
-------------------------------------------------------------------------------
-- purpose: Invalid excption
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
invalid_proc: process (clk, rst)
begin  -- process invalid_proc
  if rst = '0' then                     -- asynchronous reset (active low)

    Exceptions.invalid <= '0';
    
  elsif clk'event and clk = '1' then    -- rising clock edge

    Exceptions.invalid <= ResType.SNAN;

  end if;
end process invalid_proc;
-------------------------------------------------------------------------------
end excp_gen_beh;
