-------------------------------------------------------------------------------
-- Title      :  FPU package
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : fpu_pkg.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/11/03
-- Last update : 2000/11/03
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164
-------------------------------------------------------------------------------
-- Description: FPU core support package
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   3rd Nov 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package fpu_pkg is
-------------------------------------------------------------------------------
-- Constants

  constant SINGLE_PRECISION : integer := 32;  -- Single Precision defined size
  constant DOUBLE_PRECISION : integer := 64;  -- Double Precision defined size

  constant PRECISION : integer := SINGLE_PRECISION;  -- Selected Precision

  constant EXP           : integer                      := 8;  -- Exponent size
  constant FRACT         : integer                      := 23;  -- Fractioan size
  constant EXTENDED_SIZE : integer                      := 41;  -- fraction to extend
  -- size = 1 (carry bit)+ 1 (hidden bit)+ 23 (fract)+2 (guard & round) + 37
  -- (sticky) == 64 bits for extended fraction size
  
  constant ZERO_MODE     : std_logic_vector(1 downto 0) := "00"; 
  constant EVEN_MODE     : std_logic_vector(1 downto 0) := "01"; 
  constant POS_MODE      : std_logic_vector(1 downto 0) := "10"; 
  constant NEG_MODE      : std_logic_vector(1 downto 0) := "11"; 

  constant OPERATIONS_WIDTH : integer                      := 2;  -- Operations
  constant ADD_OP     : std_logic_vector(1 downto 0) := "00";  -- Add Operation
  constant SUB_OP     : std_logic_vector(1 downto 0) := "01";  -- Subtract operation
  constant MUL_OP     : std_logic_vector(1 downto 0) := "10";
                                        -- Multiplication operation
  constant DIV_OP     : std_logic_vector(1 downto 0) := "11";
                                        -- Division Operation

  constant DECODE_CYCLES   : integer := 1;  -- Decoded cycles
  constant ROUND_CYCLES    : integer := 6;  -- Round normalize cycles
  constant ENCODE_CYCLES   : integer := 1;  -- Encode cycles
  constant OVERHEAD_CYCLES : integer := DECODE_CYCLES+ROUND_CYCLES+ENCODE_CYCLES;
                                            -- Overhead cycles

  constant ADD_CYCLES : integer := OVERHEAD_CYCLES +3;  -- Add unit cycles
  constant SUB_CYCLES : integer := OVERHEAD_CYCLES +3;  -- subtract unit cycles

  constant MAX_CYCLES : integer := 32;  -- Maximum calculation cycles
-------------------------------------------------------------------------------
-- Types

  type FPNUMBER_typ is record           -- Floating Point number type
                         sign  : std_logic;  -- Sign of the Floating point number
                         exp   : std_logic_vector(EXP -1 downto 0);
                                        -- Floating point exponent
                         fract : std_logic_vector(FRACT -1 downto 0);
                                        -- Floating Point fraction
                       end record;


  type FPinternal_typ is record
                                        -- Floating Point internal representation (needed by all operations)
                           sign  : std_logic;  -- Sign of the number
                           exp   : std_logic_vector(EXP downto 0);
                                        -- Exponent (increased by one bit)
                           fract : std_logic_vector(FRACT+1 downto 0);  -- 25
                                        -- Fractional part (increased by 2 for
                                        -- hidden bit and carry bit)

                         end record;


  type FPRESULT_typ is record                -- Calculations result type
                         sign  : std_logic;  -- Sign of the number
                         exp   : std_logic_vector(EXP downto 0);
                                             -- Exponent (increased by one bit)
                         fract : std_logic_vector(FRACT+EXTENDED_SIZE -1 downto 0);

                       end record;


  type EXCEPTION_typ is record                     -- IEEE-754 exceptions type
                          OverFlow   : std_logic;  -- OverFlow Exception
                          UnderFlow  : std_logic;  -- UnderFlow Exception
                          DivideZero : std_logic;  -- Divide by zero Exception
                          Invalid    : std_logic;  -- Invalid Exception
                          Inexact    : std_logic;  -- Inexact Exception
                        end record;

  type CONDITIONS_typ is record               -- Condition results
                           AltB : std_logic;  -- A > B
                           BltA : std_logic;  -- B > A
                           AeqB : std_logic;  -- A = B
                         end record;


  -- The information is Active High
  -- when all are 0 thie means the number is normal
  type SPECIALNO_typ is record               -- Special No.s type
                          INF  : std_logic;  -- Infinity
                          ZERO : std_logic;  -- Zero
                          SNAN : std_logic;  -- SNAN
                          QNAN : std_logic;  -- QNAN

                        end record;

-------------------------------------------------------------------------------
-- Function Decleration

  function setQNAN

    return FPNUMBER_typ;

  function setSNAN

    return FPNUMBER_typ;

  function set_Pinf

    return FPNUMBER_typ;

  function set_Ninf

    return FPNUMBER_typ;

-------------------------------------------------------------------------------
end fpu_pkg;
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

package body fpu_pkg is

-----------------------------------------------------------------------------  
  function setQNAN

    return FPNUMBER_typ is
    variable number : FPNUMBER_typ;     -- number to set 
  begin

    number.fract := (others => '0');
    number.exp   := (others => '1');
    number.sign  := '1';

    return number;

  end setQNAN;
-----------------------------------------------------------------------------  
  function setSNAN

    return FPNUMBER_typ is
    variable number : FPNUMBER_typ;     -- number to set 
  begin

    number.fract := (others => '1');
    number.exp   := (others => '1');
    number.sign  := '1';

    return number;

  end setSNAN;

-------------------------------------------------------------------------------
  function set_Pinf

    return FPNUMBER_typ is
    variable number : FPNUMBER_typ;     -- number to set
  begin

    number.fract := (others => '0');
    number.exp   := (others => '1');
    number.sign  := '0';

    return number;

  end set_Pinf;

----------------------------------------------------------------------------- 
  function set_Ninf

    return FPNUMBER_typ is

    variable number : FPNUMBER_typ;     -- number to set

  begin

    number.fract := (others => '0');
    number.exp   := (others => '1');
    number.sign  := '1';

    return number;

  end set_Ninf;
-------------------------------------------------------------------------------

end fpu_pkg;
