-------------------------------------------------------------------------------
-- Title      :  Normalizer Rounder core
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : rndnorm.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/11/12
-- Last update : 2000/11/12
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: Leonardo/WindowsNT
-- Target      : 
-- Dependency  : ieee.std_logic_1164, ieee.std_logic_unsigned
--               utility.tools_pkg,  utility.cores_pkg
--               fpu.fpu_pkg
-------------------------------------------------------------------------------
-- Description: Adder subtractor core
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   12 Nov 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :   Denormalized number must be handled
-- To Optimze      :   Denormalized no. handling
--                     Lot of registers and logic are used
--                     Need to find a new approach to handle this block
-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   2
-- Version         :   0.11
-- Date            :   4 Dec 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Code revision
-- Known bugs      :  
--                     Inexact => whenever a round up is used (specally in zero mode)
--                     Code must be cleaned and made generic (reusable)
--					   Round to Nearest even must be fixed
--					   All rounding modes must be checked
-- To Optimze      :   Denormalized no. handling
--                     Lot of registers and logic are used
--                     Need to find a new approach to handle this block
-------------------------------------------------------------------------------


library FPU;
use FPU.fpu_pkg.all;

library utility;
use utility.tools_pkg.all;
use utility.cores_pkg.all;


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity rndnorm_ent is

  port (
    clk         : in  std_logic;        -- system clock
    rst         : in  std_logic;        -- system reset
    roundmode   : in  std_logic_vector(1 downto 0);  -- Selected Round mode
    ResIn       : in  FPRESULT_typ;     -- input reseult
    Inexact_exp : out std_logic;        -- Inexact exception signal

    Result : out FPinternal_typ);       -- result in internal format

end rndnorm_ent;

-------------------------------------------------------------------------------
architecture rndnorm_beh of rndnorm_ent is

                                           -- No. of leading zeros to post shifter
  signal ToPreShift       : FPRESULT_typ;  -- No. to preshift
  signal ToRound          : FPRESULT_typ;  -- Pre-shifted no. to be rounded

  signal Rounded_res : FPinternal_typ;  -- correct rounded result

  signal ToPostShift : FPinternal_typ;  -- To Post shift

  signal Rounded_Even : FPinternal_typ;  -- Rounded to nearest even
  signal Rounded_zero : FPinternal_typ;  -- Rounded to zero
  signal Rounded_neg  : FPinternal_typ;  -- Round to negative infinity
  signal Rounded_pos  : FPinternal_typ;  -- Round to positive infinity


  signal Inexact_Even : std_logic;      -- Inexact result from round to nearest
  signal Inexact_pos  : std_logic;      -- Inexact result from positive
  signal Inexact_neg  : std_logic;      -- Inexact result from round to negative
  signal Inexact_Zero : std_logic;      -- Inexact result from round to zero.

  signal Inexact_res  : std_logic;      -- Inexact result
  signal Inexact_exp1 : std_logic;      -- Inexact signal (internal)

  -- signal LeadZeroPre : std_logic_vector(log2(ResIn.fract)-1 downto 0);
  -- No of leading zerors in pre normalized no.

  signal NoLeadZeros      : std_logic_vector(log2(ResIn.fract'length)-1 downto 0);
                                           -- No. of leading zeros
  signal NoLeadZeros_post : std_logic_vector(log2(ToPostShift.fract'length)-1 downto 0);


begin  -- rndnorm_beh
-------------------------------------------------------------------------------
  -- purpose: Detect Leading zeros
  -- type   : sequential
  -- inputs : clk,rst
  -- outputs: 
  LeadDetect_proc : process (clk, rst)

    variable LeadZeroPre : integer range 0 to ResIn.fract'length-1;

  begin  -- process LeadDetect_proc

    if rst = '0' then                   -- asynchronous reset (active low)

      LeadZeroPre      := 0;
      NoLeadZeros      <= (others => '0');
      ToPreShift.exp   <= (others => '0');
      ToPreShift.fract <= (others => '0');
      ToPreShift.sign  <= '0';



    elsif clk'event and clk = '1' then  -- rising clock edge

      LeadZeroPre := 0;

      for i in 0 to ResIn.fract'length-2 loop

        if ResIn.fract(i) = '1' then
                                        --  64 -1 -1 = 62       
          LeadZeroPre := ResIn.fract'length-2 - i;

        end if;

      end loop;


--    case ResIn.fract is
--      when "1                         ---------------------------------------------------------------" => LeadZeroPre := 0;
--      when "01                        --------------------------------------------------------------" => LeadZeroPre := 0;
--      when "001                       -------------------------------------------------------------" => LeadZeroPre := 1;
--      when "0001                      ------------------------------------------------------------" => LeadZeroPre := 2;
--      when "00001                     -----------------------------------------------------------" => LeadZeroPre := 3;
--      when "000001                    ----------------------------------------------------------" => LeadZeroPre := 4;
--      when "0000001                   ---------------------------------------------------------" => LeadZeroPre := 5;
--      when "00000001                  --------------------------------------------------------" => LeadZeroPre := 6;
--      when "000000001                 -------------------------------------------------------" => LeadZeroPre := 7;
--      when "0000000001                ------------------------------------------------------" => LeadZeroPre := 8;
--      when "00000000001               -----------------------------------------------------" => LeadZeroPre := 9 ;
--      when "000000000001              ----------------------------------------------------" => LeadZeroPre := 10;
--      when "0000000000001             ---------------------------------------------------" => LeadZeroPre := 11;
--      when "00000000000001            --------------------------------------------------" => LeadZeroPre := 12;
--      when "000000000000001           -------------------------------------------------" => LeadZeroPre := 13;
--      when "0000000000000001          ------------------------------------------------" => LeadZeroPre := 14;
--      when "00000000000000001         -----------------------------------------------" => LeadZeroPre := 15;
--      when "000000000000000001        ----------------------------------------------" => LeadZeroPre := 16;
--      when "0000000000000000001       ---------------------------------------------" => LeadZeroPre := 17;
--      when "00000000000000000001      --------------------------------------------" => LeadZeroPre := 18;
--      when "000000000000000000001     -------------------------------------------" => LeadZeroPre := 19;
--      when "0000000000000000000001    ------------------------------------------" => LeadZeroPre := 20;
--      when "00000000000000000000001   -----------------------------------------" => LeadZeroPre := 21;
--      when "000000000000000000000001  ----------------------------------------" => LeadZeroPre := 22;
--      when "0000000000000000000000001  ---------------------------------------" => LeadZeroPre := 23;
--      when "00000000000000000000000001  --------------------------------------" => LeadZeroPre := 24;
--      when "000000000000000000000000001  -------------------------------------" => LeadZeroPre := 25;
--      when "0000000000000000000000000001  ------------------------------------" => LeadZeroPre := 26;
--      when "00000000000000000000000000001  -----------------------------------" => LeadZeroPre := 27;
--      when "000000000000000000000000000001  ----------------------------------" => LeadZeroPre := 28;
--      when "0000000000000000000000000000001  ---------------------------------" => LeadZeroPre := 29;
--      when "00000000000000000000000000000001  --------------------------------" => LeadZeroPre := 30;
--      when "000000000000000000000000000000001  -------------------------------" => LeadZeroPre := 31;
--      when "0000000000000000000000000000000001  ------------------------------" => LeadZeroPre := 32;
--      when "00000000000000000000000000000000001  -----------------------------" => LeadZeroPre := 33;
--      when "000000000000000000000000000000000001  ----------------------------" => LeadZeroPre := 34;
--      when "0000000000000000000000000000000000001  ---------------------------" => LeadZeroPre := 35;
--      when "00000000000000000000000000000000000001  --------------------------" => LeadZeroPre := 36;
--      when "000000000000000000000000000000000000001  -------------------------" => LeadZeroPre := 37;
--      when "0000000000000000000000000000000000000001  ------------------------" => LeadZeroPre := 38;
--      when "00000000000000000000000000000000000000001  -----------------------" => LeadZeroPre := 39;
--      when "000000000000000000000000000000000000000001  ----------------------" => LeadZeroPre := 40;
--      when "0000000000000000000000000000000000000000001  ---------------------" => LeadZeroPre := 41;
--      when "00000000000000000000000000000000000000000001  --------------------" => LeadZeroPre := 42;
--      when "000000000000000000000000000000000000000000001  -------------------" => LeadZeroPre := 43;
--      when "0000000000000000000000000000000000000000000001  ------------------" => LeadZeroPre := 44;
--      when "00000000000000000000000000000000000000000000001  -----------------" => LeadZeroPre := 45;
--      when "000000000000000000000000000000000000000000000001  ----------------" => LeadZeroPre := 46;
--      when "0000000000000000000000000000000000000000000000001  ---------------" => LeadZeroPre := 47;
--      when "00000000000000000000000000000000000000000000000001  --------------" => LeadZeroPre := 48;
--      when "000000000000000000000000000000000000000000000000001  -------------" => LeadZeroPre := 49;
--      when "0000000000000000000000000000000000000000000000000001  ------------" => LeadZeroPre := 50;
--      when "00000000000000000000000000000000000000000000000000001  -----------" => LeadZeroPre := 51;
--      when "000000000000000000000000000000000000000000000000000001  ----------" => LeadZeroPre := 52;
--      when "0000000000000000000000000000000000000000000000000000001  ---------" => LeadZeroPre := 53;
--      when "00000000000000000000000000000000000000000000000000000001  --------" => LeadZeroPre := 54;
--      when "000000000000000000000000000000000000000000000000000000001  -------" => LeadZeroPre := 55;
--      when "0000000000000000000000000000000000000000000000000000000001  ------" => LeadZeroPre := 56;
--      when "00000000000000000000000000000000000000000000000000000000001  -----" => LeadZeroPre := 57;
--      when "000000000000000000000000000000000000000000000000000000000001  ----" => LeadZeroPre := 58;
--      when "0000000000000000000000000000000000000000000000000000000000001  ---" => LeadZeroPre := 59;
--      when "00000000000000000000000000000000000000000000000000000000000001  --" => LeadZeroPre := 60;  
--      when "000000000000000000000000000000000000000000000000000000000000001-" => LeadZeroPre := 61;
--      when "0000000000000000000000000000000000000000000000000000000000000001" => LeadZeroPre := 62;
--      when "0000000000000000000000000000000000000000000000000000000000000000" => LeadZeroPre := 63;
--
--      when others => LeadZeroPre := 0;
--    end case;

-- Converting integer to std_logic_vector
      NoLeadZeros <= int_2_slv(LeadZeroPre, NoLeadZeros'length);

      ToPreShift.exp   <= ResIn.exp;
      ToPreShift.fract <= ResIn.fract;
      ToPreShift.sign  <= ResIn.sign;

    end if;
  end process LeadDetect_proc;
-------------------------------------------------------------------------------
  -- purpose: Pre-shifter
  -- type   : sequential
  -- inputs : clk, rst
  -- outputs: 
  pre_shift      : process (clk, rst)
    variable exp : SIGN_MAG_typ(ResIN.exp'length downto 0);

  begin  -- process pre_shift
    if rst = '0' then                   -- asynchronous reset (active low)

      ToRound.fract <= (others => '0');
      ToRound.sign  <= '0';
      ToRound.exp   <= (others => '0');

    elsif clk'event and clk = '1' then  -- rising clock edge

      -- If the MSB of the fraction is 1 then shift right one bit
      if ToPreShift.fract(ToPreShift.fract'length -1 ) = '1' then

        ToRound.fract <= '0' & ToPreShift.fract(ToPreShift.fract'length-1 downto 1);
        ToRound.sign  <= ToPreShift.sign;
        exp           := SIGN_MAG_typ('0' & ToPreShift.exp) + "0000000001";


        ToRound.exp <= std_logic_vector(exp(exp'length-2 downto 0));

        -- If the MSB is not 1 then possible shift to Left
      else

        ToRound.sign <= ToPreShift.sign;
--'0' & (ToPreShift.exp'length - NoLeadZeros'length -1 downto NoLeadZeros'length => '0')
        exp          := SIGN_MAG_typ('0' & ToPreShift.exp) - SIGN_MAG_typ("000" & NoLeadZeros);

        -- Denormalized No. Handler
        if exp(exp'length -1) = '1' then  -- check if the exponent is negative
                                          -- this means we have a denormalized no.

          -- if negative result do shift on the min operand which is exp in
          -- this case. For example exp = 50 & nozeros = 63 =>
          -- shiftwidth = 50 -63 = - 13 ==> shift toward zero exponent = 50
          ToRound.fract <= LeftShift (ToPreShift.fract, ToPreShift.exp(NoLeadZeros'length-1 downto 0));
          ToRound.exp   <= (others => '0');

        else

          ToRound.fract <= LeftShift (ToPreShift.fract, NoLeadZeros);
          ToRound.exp   <= std_logic_vector(exp(exp'length-2 downto 0));

        end if;  -- neg exponent diff

      end if;  -- shift right or left

    end if;  --clk
  end process pre_shift;

-------------------------------------------------------------------------------
-- purpose: Round to negative Infinity
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  rnd_neg_proc : process (clk, rst)

    variable roundup : std_logic;       -- Round up status
    variable s_bit   : std_logic;       -- Sticky bit
    variable r_bit   : std_logic;       -- Round bit

  begin  -- process rnd_neg_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      Rounded_neg.sign  <= '0';
      Rounded_neg.exp   <= (others => '0');
      Rounded_neg.fract <= (others => '0');

      roundup := '0';
      s_bit   := '0';
      r_bit   := '0';

      Inexact_neg <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge

      s_bit   := '0';
      -- must be checked
      for i in 0 to ToRound.fract'length - Rounded_neg.fract'length -1 -1 -1 -1 loop
        s_bit := s_bit or ToRound.fract(i);
      end loop;  -- i
      -- [MSB:g:r:ssssss...] 
      r_bit   := ToRound.fract(ToRound.fract'length - Rounded_neg.fract'length -1 -1 -1 );
      --must be checked

      roundup := ToRound.sign and (s_bit or r_bit);

      Rounded_neg.sign <= ToRound.sign;
      Rounded_neg.exp  <= ToRound.exp(ToRound.exp'length -1 downto 0);
      -- MSB is used for overflow detection

      if roundup = '1' then
        Inexact_neg <= '1';


        Rounded_neg.fract <= ToRound.fract(63 downto 39) + "0000000000000000000000001";

      else
        Inexact_neg       <= '0';
        Rounded_neg.fract <= ToRound.fract(63 downto 39);  --ToRound.fract(ToRound.fract'length-1 downto ToRound.fract'length - Rounded_zero.fract'length );
      end if;


    end if;
  end process rnd_neg_proc;
-------------------------------------------------------------------------------
-- purpose: Round to positive Infinity
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  rnd_POS_proc : process (clk, rst)

    variable roundup : std_logic;       -- Round up status
    variable s_bit   : std_logic;       -- Sticky bit
    variable r_bit   : std_logic;       -- Round bit

  begin  -- process rnd_pos_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      Rounded_pos.sign  <= '0';
      Rounded_pos.exp   <= (others => '0');
      Rounded_pos.fract <= (others => '0');

      roundup := '0';
      s_bit   := '0';
      r_bit   := '0';

      Inexact_pos <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge

      s_bit   := '0';
      -- must be checked
      for i in 0 to ToRound.fract'length - Rounded_pos.fract'length -1 -1 -1-1 loop
        s_bit := s_bit or ToRound.fract(i);
      end loop;  -- i
      -- [MSB:g:r:ssssss...] 
      r_bit   := ToRound.fract(ToRound.fract'length - Rounded_pos.fract'length -1 -1 -1 );  --
      --must be checked

      roundup := (not ToRound.sign) and (s_bit or r_bit);

      Rounded_pos.sign <= ToRound.sign;
      Rounded_pos.exp  <= ToRound.exp(ToRound.exp'length -1 downto 0);
      -- MSB is used for overflow detection

      if roundup = '1' then
        Inexact_pos       <= '1';
--        Rounded_pos.fract <= ToRound.fract(ToRound.fract'length -1 downto ToRound.fract'length - Rounded_pos.fract'length -3) + "0000000000000000000000001";
        Rounded_pos.fract <= ToRound.fract(63 downto 39) + "0000000000000000000000001";

-- must set inexact flag
      else
        Inexact_pos       <= '0';
        --  Rounded_pos.fract <= ToRound.fract(ToRound.fract'length -1 downto ToRound.fract'length - Rounded_pos.fract'length -3);
        Rounded_pos.fract <= ToRound.fract(63 downto 39);
      end if;


    end if;
  end process rnd_pos_proc;
-------------------------------------------------------------------------------
-- purpose: Round to zero
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  rnd_zero_proc : process (clk, rst)
  begin  -- process rnd_zero_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      Rounded_zero.sign  <= '0';
      Rounded_zero.exp   <= (others => '0');
      Rounded_zero.fract <= (others => '0');

      Inexact_zero <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge

      Rounded_zero.sign <= ToRound.sign;
      Rounded_zero.exp  <= ToRound.exp(ToRound.exp'length -1 downto 0);
      -- MSB is used for overflow detection


      Rounded_zero.fract <= ToRound.fract(63 downto 39);  --ToRound.fract(ToRound.fract'length-1 downto ToRound.fract'length - Rounded_zero.fract'length );
---------
---------
-- ======
-- ======
-- ======
---------
---------
      -- must be like stickey bits

      Inexact_Zero <=ToRound.fract(38);
    end if;
  end process rnd_zero_proc;
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- purpose: Round to nearest even
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  rnd_even_proc : process (clk, rst)

    variable roundup : std_logic;       -- Round up status
    variable s_bit   : std_logic;       -- Sticky bit
    variable r_bit   : std_logic;       -- Round bit

  begin  -- process rnd_even_proc
    if rst = '0' then                   -- asynchronous reset (active low)

      Rounded_even.sign  <= '0';
      Rounded_even.exp   <= (others => '0');
      Rounded_even.fract <= (others => '0');

      roundup := '0';
      s_bit   := '0';
      r_bit   := '0';

      Inexact_even <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge


      s_bit   := '0';
      -- must be checked
      for i in 0 to ToRound.fract'length - Rounded_pos.fract'length -1 -1 -1-1 loop
        s_bit := s_bit or ToRound.fract(i);
      end loop;  -- i
      -- [MSB:g:r:ssssss...] 
      r_bit   := ToRound.fract(ToRound.fract'length - Rounded_even.fract'length -1 -1 -1 );  --
      --must be checked

      roundup := (not ToRound.sign) and (s_bit or r_bit);

      Rounded_even.sign <= ToRound.sign;
      Rounded_even.exp  <= ToRound.exp(ToRound.exp'length -1 downto 0);


      if roundup = '1' then
        Inexact_even       <= '1';
--        Rounded_pos.fract <= ToRound.fract(ToRound.fract'length -1 downto ToRound.fract'length - Rounded_pos.fract'length -3) + "0000000000000000000000001";
        Rounded_even.fract <= ToRound.fract(63 downto 39) + "0000000000000000000000001";

-- must set inexact flag
      else
        Inexact_even       <= '0';
        --  Rounded_pos.fract <= ToRound.fract(ToRound.fract'length -1 downto ToRound.fract'length - Rounded_pos.fract'length -3);
        Rounded_even.fract <= ToRound.fract(63 downto 39);
      end if;


    end if;
  end process rnd_even_proc;
-------------------------------------------------------------------------------

-- purpose: Select rounded result
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  select_res : process (clk, rst)
  begin  -- process select_res
    if rst = '0' then                   -- asynchronous reset (active low)
      rounded_res.sign      <= '0';
      rounded_res.exp       <= (others => '0');
      rounded_res.fract     <= (others => '0');
      Inexact_res           <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      case roundmode is
        -- round to zero
        when ZERO_MODE                 =>
          rounded_res.sign  <= rounded_zero.sign;
          rounded_res.exp   <= rounded_zero.exp;
          rounded_res.fract <= rounded_zero.fract;
          Inexact_res       <= Inexact_zero;

          -- round to nearest even
        when EVEN_MODE =>
          rounded_res.sign  <= rounded_even.sign;
          rounded_res.exp   <= rounded_even.exp;
          rounded_res.fract <= rounded_even.fract;
          Inexact_res       <= Inexact_even;
          -- round to positive
        when POS_MODE  =>
          rounded_res.sign  <= rounded_pos.sign;
          rounded_res.exp   <= rounded_pos.exp;
          rounded_res.fract <= rounded_pos.fract;
          Inexact_res       <= Inexact_pos;
          -- round to negative
        when NEG_MODE  =>
          rounded_res.sign  <= rounded_neg.sign;
          rounded_res.exp   <= rounded_neg.exp;
          rounded_res.fract <= rounded_neg.fract;
          Inexact_res       <= Inexact_neg;
        when others    =>
          rounded_res.sign  <= rounded_zero.sign;
          rounded_res.exp   <= rounded_zero.exp;
          rounded_res.fract <= rounded_zero.fract;
          Inexact_res       <= Inexact_zero;

      end case;

    end if;
  end process select_res;
-------------------------------------------------------------------------------
-- purpose: Detect leading zeros
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  LeadDetect_post         : process (clk, rst)
    variable LeadZeroPost : integer range 0 to rounded_res.fract'length-1;
  begin  -- process LeadDetect_post
    if rst = '0' then                   -- asynchronous reset (active low)

      LeadZeroPost      := 0;
      NoLeadZeros_post  <= (others => '0');
      ToPostShift.exp   <= (others => '0');
      ToPostShift.fract <= (others => '0');
      ToPostShift.sign  <= '0';

      Inexact_exp1 <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge
      LeadZeroPost := 0;

      for i in 0 to rounded_res.fract'length-2 loop

        if rounded_res.fract(i) = '1' then

          LeadZeroPost := rounded_res.fract'length-2 - i;

        end if;

      end loop;
-- Converting integer to std_logic_vector
      NoLeadZeros_post <= int_2_slv(LeadZeroPost, NoLeadZeros_post'length);

      ToPostShift.exp   <= rounded_res.exp;
      ToPostShift.fract <= rounded_res.fract;
      ToPostShift.sign  <= rounded_res.sign;

      Inexact_exp1 <= Inexact_res;
    end if;
  end process LeadDetect_post;
-------------------------------------------------------------------------------
-- purpose: Post shift process and final one
-- type   : sequential
-- inputs : clk, rst
-- outputs: 
  PostShift : process (clk, rst)

    variable exp : SIGN_MAG_typ(ToPostShift.exp'length downto 0);

  begin  -- process PostShift
    if rst = '0' then                   -- asynchronous reset (active low)

      Result.fract <= (others => '0');
      Result.sign  <= '0';
      Result.exp   <= (others => '0');
      exp           := (others => '0');

      Inexact_exp <= '0';

    elsif clk'event and clk = '1' then  -- rising clock edge


      -- If the MSB of the fraction is 1 then shift right one bit
      if ToPostShift.fract(ToPostShift.fract'length -1 ) = '1' then

        Result.fract <= '0' & ToPostShift.fract(ToPostShift.fract'length-1 downto 1);
        Result.sign  <= ToPostShift.sign;
        exp           := SIGN_MAG_typ('0' & ToPostShift.exp) + "0000000001";


        Result.exp <= std_logic_vector(exp(exp'length-2 downto 0));

        -- If the MSB is not 1 then possible shift to Left
      else

        Result.sign <= ToPostShift.sign;
--'0' & (ToPreShift.exp'length - NoLeadZeros'length -1 downto NoLeadZeros'length => '0')
        exp          := SIGN_MAG_typ('0' & ToPostShift.exp) - SIGN_MAG_typ("00000" & NoLeadZeros_Post);

        -- Denormalized No. Handler
        if exp(exp'length -1) = '1' then  -- check if the exponent is negative
                                          -- this means we have a denormalized no.

          -- if negative result do shift on the min operand which is exp in
          -- this case. For example exp = 50 & nozeros = 63 =>
          -- shiftwidth = 50 -63 = - 13 ==> shift toward zero exponent = 50
          Result.fract <= LeftShift (ToPostShift.fract, ToPostShift.exp(NoLeadZeros_post'length-1 downto 0));
          Result.exp   <= (others => '0');

        else

          Result.fract <= LeftShift (ToPostShift.fract, NoLeadZeros_post);
          Result.exp   <= std_logic_vector(exp(exp'length-2 downto 0));

        end if;  -- neg exponent diff

      end if;  -- shift right or left

      Inexact_exp <= Inexact_exp1;

    end if;
  end process PostShift;
-------------------------------------------------------------------------------


-------------------------------------------------------------------------------
end rndnorm_beh;

