-------------------------------------------------------------------------------
-- Title      :  FPU client Test Bench
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : Fpu_client_tb.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/12/18
-- Last update: 2001/02/03
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: 
-- Target      : 
-- Dependency  : ieee.std_logic_1164
--               fpu.fpu_pkg
--               fpu_tb.fpu_tb_pkg
--               std.textio
--               TextLib.txt_util
--               simLib.sim_utils_pkg
--               FIO.PCK_FIO
-------------------------------------------------------------------------------
-- Description: FPU client test bench.
--              It reads the test vectors file
--              Send it to Server Test bench
--              checks the result from the server test bench 
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   18 Dec 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :   
-- To Optimize
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use std.textio.all;

library TextLib;
use TextLib.txt_util.all;

library simLib;
use simLib.sim_utils_pkg.all;

library fpu_tb;
use fpu_tb.fpu_tb_pkg.all;

library FIO;
use FIO.PCK_FIO.all;

entity Fpu_client_tb_ent is
  generic (
    LogFile  :     string := "SimResult.log";  -- Log File name
    StimFile :     string := "stim.vec");      -- File Name
  port (
    FPUTask  : out FPUTask_typ;                -- FPU task to the server FPU TB
    EOT      : in  std_logic;                  -- End of FPU Task
    Result   : in  FPUTaskResult_typ);         -- FPU Task Result

end Fpu_client_tb_ent;
-------------------------------------------------------------------------------

architecture FPU_client_tb_beh of Fpu_client_tb_ent is

--  file DataPattern : TEXT is in StimFile;  -- Data Pattern (Test vectors) vhdl87


  file DataPattern : text open read_mode is StimFile;  -- Data Pattern (Test
                                                       -- vectors) vhdl93

--  file Log : text open write_mode is LogFile;  -- Log File  vhdl93
  file Log : text is out LogFile;                -- Log File  vhdl87

begin  -- FPU_client_tb

  -- purpose: Get new task and check old one.
  -- type   : combinational
  -- inputs : EOT
  -- outputs: 
  process (EOT)

    variable data : line;               -- data line
    variable exp  : integer;            -- exponent

-------------------------------------------------------------------------------
-- Expected results    
    variable expected_fract : std_logic_vector( 22 downto 0);
                                                              -- Expected Fraction
    variable expected_exp   : std_logic_vector( 7 downto 0);  -- Expexted Exp
    variable expected_sign  : std_logic;                      -- Expeccted sign

    variable div_zero  : std_logic;     -- Divided by zero exception
    variable overflow  : std_logic;     -- Overflow
    variable underflow : std_logic;     -- underflow
    variable invalid   : std_logic;     -- Invalid
    variable inexact   : std_logic;     -- Inexact


    variable Operand1_str   : string(1 to 8);  -- Operand1
    variable Operand2_str   : string(1 to 8);  -- Operand2
    variable Exceptions_str : string(1 to 1);  -- Exceptions
    variable compare_str    : string(1 to 1);  -- Compare results
    variable result_str     : string(1 to 8);  -- Result
    variable Operation_str  : string(1 to 1);  -- Operation
    variable RoundMode_str  : string(1 to 1);  -- Round mode

    variable str_line     : string (1 to 28);  -- String Line
    variable log_str_line : line;              -- Log Line

    variable counter : integer;         -- General purpose Counter

    variable Operand1_slv : std_logic_vector(31 downto 0);  -- Operand 1
    variable Operand2_slv : std_logic_vector(31 downto 0);  -- Operand 2
    variable Result_slv   : std_logic_vector(31 downto 0);  -- Result

    variable valid : boolean := true;   -- flag

  begin  -- process
    if endfile(DataPattern) then
      FILE_CLOSE(DataPattern);
      FILE_OPEN(DataPattern, stimFile);
    end if;

    if EOT'event and EOT = '1' then
      valid                       := true;
--      Check Old result
      fprint(log, log_str_line, " ===========================\n");
      if expected_fract /= Result.Result.fract then
        fprint(log, log_str_line,
               "[ERROR!!] Invalid Fraction result [%r] for \n"&
               "Operands [%r] and [%r] Expected[%r]\n",
               fo(Result.Result.fract), fo(Operand1_slv), fo(Operand2_slv), fo(expected_fract));
        valid                     := false;
      end if;

      if expected_exp /= Result.Result.exp then
        fprint(log, log_str_line,
               "[ERROR!!] Invalid Exponent result [%r] for \n"&
               "Operands [%r] and [%r] Expected[%r]\n",
               fo(Result.Result.exp), fo(Operand1_slv), fo(Operand2_slv), fo(expected_exp));
        valid := false;
      end if;

      if expected_sign /= Result.Result.sign then
        fprint(log, log_str_line,
               "[ERROR!!] Invalid sign result [%r] for \n"&
               "Operands [%r] and [%r] Expected[%r]\n",
               fo(Result.Result.sign), fo(Operand1_slv), fo(Operand2_slv), fo(expected_sign));
        valid := false;
      end if;

      if valid = true then
        fprint(log, log_str_line,
               "[Valid Operation] [OP1][%r] "&
               "[OP2][%r] "&
               "[Res] sign[%r]Exp[%r]Fract[%r] \n",
               fo(Operand1_slv), fo(Operand2_slv),
               fo(Result.Result.sign), fo(Result.Result.exp), fo(Result.Result.fract) );
      end if;

--  get new data pattern
      str_read(DataPattern, str_line);
-- File Format
-- Operation|Operand1|Operand2|Result  |Exceptions|RoundMode|Compare|
--  X       |XXXXXXXX|XXXXXXXX|XXXXXXXX|X         |X        |X      |
-- Where X represents a single HEX digit
--
      case str_line(1) is
        when '0'    => FPUTask.operation <= "0001";  -- addition
                       fprint(log, log_str_line, "Addition Operation\n");
        when '1'    => FPUTask.operation <= "0010";  -- subtract
                       fprint(log, log_str_line, "Subtraction Operation\n");
        when '2'    => FPUTask.operation <= "0100";  -- Multiplication
                       fprint(log, log_str_line, "non supported Operation\n");
        when '3'    => FPUTask.operation <= "1000";  -- division
                       fprint(log, log_str_line, "non supported Operation\n");
        when others => fprint(log, log_str_line, "non supported Operation\n");
      end case;

      div_zero  := '0';
      overflow  := '0';
      underflow := '0';
      invalid   := '0';
      inexact   := '0';


      case str_line(26) is
        when '0'    => null;
        when '1'    => div_zero  := '1';
        when '2'    => overflow  := '1';
        when '3'    => underflow := '1';
        when '4'    => invalid   := '1';
        when '5'    => inexact   := '1';
        when others => null;
      end case;

      case str_line(27) is
        when '1'    => FPUTask.Round_Mode <= "0001";
                       fprint(log, log_str_line, "Nearset Zero Round Mode\n");
        when '2'    => FPUTask.Round_Mode <= "0010";
                       fprint(log, log_str_line, "Nearset even Round Mode\n");
        when '4'    => FPUTask.Round_Mode <= "0100";
                       fprint(log, log_str_line, "Nearset Positive Round Mode\n");
        when '8'    => FPUTask.Round_Mode <= "1000";
                       fprint(log, log_str_line, "Nearset negative Round Mode\n");
        when others => null;
                       fprint(log, log_str_line, "non supported Round Mode\n");
      end case;


      Counter := 0;
      for i in 2 to 9 loop

        Operand1_slv(31-counter downto 28-counter) := hexstr_2_slv (str_line(i));
        Counter                                    := Counter+4;

      end loop;  -- i

      FPUTask.op1.sign  <= Operand1_slv(31);
      FPUTask.op1.exp   <= Operand1_slv(30 downto 23);
      FPUTask.op1.Fract <= Operand1_slv(22 downto 0);


      Counter := 0;
      for i in 10 to 17 loop

        Operand2_slv(31-counter downto 28-counter) := hexstr_2_slv (str_line(i));
        Counter                                    := Counter+4;

      end loop;  -- i

      FPUTask.op2.sign  <= Operand2_slv(31);
      FPUTask.op2.exp   <= Operand2_slv(30 downto 23);
      FPUTask.op2.Fract <= Operand2_slv(22 downto 0);


      Counter := 0;
      for i in 18 to 25 loop

        Result_slv(31-counter downto 28-counter) := hexstr_2_slv (str_line(i));
        Counter                                  := Counter+4;

      end loop;  -- i

      expected_sign  := Result_slv(31);
      expected_exp   := Result_slv(30 downto 23);
      expected_Fract := Result_slv(22 downto 0);


    end if;

  end process;
-------------------------------------------------------------------------------
end FPU_client_tb_beh;
