-------------------------------------------------------------------------------
-- Title      :  FPU server Test Bench
-- Project    :  Floating-Point Unit core
-------------------------------------------------------------------------------
-- File        : Fpu_server_tb.vhd
-- Author      : Jamil Khatib  (khatib@ieee.org)
-- Organization: OpenIPCore Project
-- Created     : 2000/12/20
-- Last update : 2000/12/24
-- Platform    : 
-- Simulators  : Modelsim 5.3XE/Windows98
-- Synthesizers: 
-- Target      : 
-- Dependency  : ieee.std_logic_1164
--               fpu.fpu_pkg, fpu_tb.fpu_tb_pkg
--                               fpu.fpu_pkg,  fpu.components
-------------------------------------------------------------------------------
-- Description: FPU server test bench.
--              It accepts tasks from the client TB
--              It executes the tasks according to the proper timing
--              It checks the timing of the FPU
-------------------------------------------------------------------------------
-- Copyright (c) 2000 Jamil Khatib
-- 
-- This VHDL design file is an open design; you can redistribute it and/or
-- modify it and/or implement it after contacting the author
-- You can check the draft license at
-- http://www.opencores.org/OIPC/license.shtml

-------------------------------------------------------------------------------
-- Revisions  :
-- Revision Number :   1
-- Version         :   0.1
-- Date            :   18 Dec 2000
-- Modifier        :   Jamil Khatib (khatib@ieee.org)
-- Desccription    :   Created
-- Known bugs      :   
-- To Optimize
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;

library fpu_tb;
use fpu_tb.fpu_tb_pkg.all;

library fpu;
use fpu.fpu_pkg.all;
use fpu.components_pkg.all;

entity fpu_server_tb_ent is

  port (
    FPUTask    : in    FPUTask_typ;       -- FPU task to be executed
    EOT        : inout std_logic;         -- End of current task
    TaskResult : out   FPUTaskResult_typ  -- FPU task result
-------------------------------------------------------------------------------
    );

                                        -- Operations
end fpu_server_tb_ent;
-------------------------------------------------------------------------------

architecture Fpu_server_tb_beh of fpu_server_tb_ent is

-------------------------------------------------------------------------------
-- purpose: Request for a New task
  procedure RequestNewTask(signal EOT : inout std_logic) is
  begin  -- RequestNewTask

    EOT <= '1';
    wait for 1 ns;
    EOT <= '0';

  end RequestNewTask;
-------------------------------------------------------------------------------
  signal clk        : std_logic := '0';              -- UUT clock
  signal reset      : std_logic;                     -- UUT Reset
  signal load       : std_logic;                     -- Load new operads to UUT
  signal done       : std_logic;
                                                     -- UUT end of calculation and valid result
  signal Operand1   : FPNUMBER_typ;                  -- Operand 1 to UUT
  signal Operand2   : FPNUMBER_typ;                  -- Operand 2 to UUT
  signal Round_mode : std_logic_vector(1 downto 0);  -- UUT round mode
  signal Exceptions : EXCEPTION_typ;                 -- UUT exceptions
  signal conditions : CONDITIONS_typ;                -- UUT conditions
  signal Result     : FPNUMBER_typ;                  -- UUT result
  signal Operation  : std_logic_vector(OPERATIONS_WIDTH-1 downto 0);

begin  -- Fpu_server_tb_beh
-------------------------------------------------------------------------------
  UUT : fpu_ent
    generic map (
      FP_PRECISION => SINGLE_PRECISION)
    port map (
      Operand1     => Operand1,
      Operand2     => Operand2,
      clk          => clk,
      rst          => reset,
      Round_Mode   => Round_Mode,
      Result       => Result,
      Exceptions   => Exceptions,
      conditions   => conditions,
      load         => load,
      done         => done,
      Operation    => Operation);
-------------------------------------------------------------------------------
  clk <= not clk after 20 ns;

  reset <= '0',
           '1' after 5 ns;

-------------------------------------------------------------------------------
  -- purpose: Timing Checker
  TimingChk_proc      : process
    variable NoCycles : integer := 0;
                                        -- No of cycles neded to complete the operation

  begin  -- process TimingChk_proc
    if reset = '0' then                 -- asynchronous reset (active low)

      EOT <= '0';
      wait for 1 ns;

    else

      RequestNewTask(EOT);

      wait until clk = '0';

      Operand1.fract <= FPUTask.Op1.fract;
      Operand1.exp   <= FPUTask.Op1.exp;
      Operand1.sign  <= FPUTask.Op1.sign;

      Operand2.fract <= FPUTask.Op2.fract;
      Operand2.exp   <= FPUTask.Op2.exp;
      Operand2.sign  <= FPUTask.Op2.sign;

      case FPUTask.Round_Mode is
        when "0001" => Round_mode <= ZERO_MODE;
        when "0010" => Round_mode <= EVEN_MODE;
        when "0100" => Round_mode <= POS_MODE;
        when "1000" => Round_mode <= NEG_MODE;
        when others => null;
                       -- must set and error message for non supported round
                       -- modes
      end case;

      case FPUTask.Operation is
        when "0001" => Operation <= ADD_OP;
                       NoCycles  := ADD_CYCLES;
        when "0010" => Operation <= SUB_OP;
                       NoCycles  := SUB_CYCLES;
--        when "0100" => Operation <= MUL_OP;
--        when "1000" => Operation <= DIV_OP;
        when others => null;
                                        -- Must set an error message for
                                        -- non supported operations
      end case;


      Load <= '1';
      wait until clk = '0';
      load <= '0';

      for i in 0 to NoCycles loop

        assert done = '0' report "Early Valid Result" severity warning;
        wait until clk = '1';
      end loop;  -- i

      assert done = '1' report "Late Valid Result" severity error;


      TaskResult.Result.fract <= Result.fract;
      TaskResult.Result.exp   <= Result.exp;
      TaskResult.Result.sign  <= Result.sign;

      TaskResult.Exceptions <= Exceptions;
      TaskResult.conditions <= conditions;


    end if;
  end process TimingChk_proc;

end Fpu_server_tb_beh;
